/* treetopgm - make a listing from `find' into a graphical tree.
 * Copyright (C) 1996,8 Russell Marks. See treetopgm.c for license details.
 *
 * writemrf.c - write mrf file (based on pbmtomrf).
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>


static int bitbox,bitsleft;


void bit_init()
{
bitbox=0; bitsleft=8;
}


void bit_output(int bit)
{
bitsleft--;
bitbox|=(bit<<bitsleft);
if(!bitsleft)
  {
  putchar(bitbox);
  bitbox=0;
  bitsleft=8;
  }
}


void bit_flush()
{
/* there are never 0 bits left outside of bit_output, but
 * if 8 bits are left here there's nothing to flush, so
 * only do it if bitsleft!=8.
 */
if(bitsleft!=8)
  {
  bitsleft=1;
  bit_output(0);	/* yes, really. This will always work. */
  }
}


void do_square(unsigned char *image,int ox,int oy,int w,int h,int size)
{
int x,y,t=0;

/* check square to see if it's all black or all white. */

for(y=0;y<size;y++)
  for(x=0;x<size;x++)
    if(ox+x<w && oy+y<h)
      t+=(image[(oy+y)*w+ox+x]>=128);
    else
      t++;

/* if the total's 0, it's black. if it's size*size, it's white. */
if(t==0 || t==size*size)
  {
  if(size!=1)		/* (it's implicit when size is 1, of course) */
    bit_output(1);	/* all same colour */
  bit_output(t?1:0);
  return;
  }

/* otherwise, if our square is greater than 1x1, we need to recurse. */
if(size>1)
  {
  bit_output(0);	/* not all same */
  size>>=1;
  do_square(image,ox,oy,w,h,size);
  do_square(image,ox+size,oy,w,h,size);
  do_square(image,ox,oy+size,w,h,size);
  do_square(image,ox+size,oy+size,w,h,size);
  }
}




void writemrf(unsigned char *bytemap,int w,int h)
{
int x,y,w64,h64;

printf("MRF1");
printf("%c%c%c%c",w>>24,w>>16,w>>8,w&255);
printf("%c%c%c%c",h>>24,h>>16,h>>8,h&255);
putchar(0);	/* option byte, unused for now */

/* w64 is units-of-64-bits width, h64 same for height */
w64=(w+63)/64;
h64=(h+63)/64;

/* now recursively check squares. */

/* init bit output */
bit_init();

for(y=0;y<h64;y++)
  for(x=0;x<w64;x++)
    do_square(bytemap,x*64,y*64,w,h,64);

bit_flush();
}
